/* Copyright (C) Sportradar AG. See LICENSE for full license governing this code */
package com.sportradar.unifiedodds.example.markets;

import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.AbstractMap.SimpleEntry;

import com.sportradar.unifiedodds.sdk.MarketDescriptionManager;
import com.sportradar.unifiedodds.sdk.MarketDescriptionManager.MarketMapping;
import com.sportradar.unifiedodds.sdk.MessageInterest;
import com.sportradar.unifiedodds.sdk.OddsFeedConfiguration;
import com.sportradar.unifiedodds.sdk.OddsFeedListener;
import com.sportradar.unifiedodds.sdk.OddsFeedSession;
import com.sportradar.unifiedodds.sdk.OddsFeedSessionBuilder;
import com.sportradar.unifiedodds.sdk.SDKGlobalEventsListener;
import com.sportradar.unifiedodds.sdk.oddsentities.BetCancel;
import com.sportradar.unifiedodds.sdk.oddsentities.BetSettlement;
import com.sportradar.unifiedodds.sdk.oddsentities.BetStop;
import com.sportradar.unifiedodds.sdk.oddsentities.FixtureChange;
import com.sportradar.unifiedodds.sdk.oddsentities.Market;
import com.sportradar.unifiedodds.sdk.oddsentities.OddsChange;
import com.sportradar.unifiedodds.sdk.oddsentities.Outcome;
import com.sportradar.unifiedodds.sdk.oddsentities.Product;
import com.sportradar.unifiedodds.sdk.oddsentities.ProductDown;
import com.sportradar.unifiedodds.sdk.oddsentities.RollbackBetCancel;
import com.sportradar.unifiedodds.sdk.oddsentities.RollbackBetSettlement;
import com.sportradar.unifiedodds.sdk.sportentities.SportEvent;

/**
 * The following example prints all available markets and how they are mapped to other products.
 * 
 * Usage: java -Duf.accesstoken=<your-access-token> MarketList
 *
 */
public class MarketList {

    static final class Dummy implements SDKGlobalEventsListener, OddsFeedListener {
        @Override
        public void onProductDown(ProductDown productDown) {}

        @Override
        public void onConnectionDown() {}

        @Override
        public void onOddsChange(OddsFeedSession sender, OddsChange<SportEvent> oddsChanges) {}

        @Override
        public void onBetStop(OddsFeedSession sender, BetStop<SportEvent> betStop) {}

        @Override
        public void onBetSettlement(OddsFeedSession sender, BetSettlement<SportEvent> clearBets) {}

        @Override
        public void onRollbackBetSettlement(OddsFeedSession sender,
                    RollbackBetSettlement<SportEvent> rollbackBetSettlement) {}

        @Override
        public void onBetCancel(OddsFeedSession sender, BetCancel<SportEvent> betCancel) {}

        @Override
        public void onRollbackBetCancel(OddsFeedSession sender, RollbackBetCancel<SportEvent> rbBetCancel) {}

        @Override
        public void onFixtureChange(OddsFeedSession sender, FixtureChange<SportEvent> fixtureChange) {}

        @Override
        public void onUnparseableMessage(OddsFeedSession sender, byte[] rawMessage, SportEvent event) {
            // TODO Auto-generated method stub

        }
    }

    public static void main(String[] args) throws Exception {
        boolean includeOutcomes = false;
        OddsFeedConfiguration config = new OddsFeedConfiguration();
        Dummy dummy = new Dummy();
        config.setProductEnabled(Product.LCoO, false);
        config.setProductEnabled(Product.LiveOdds, false);
        OddsFeedSession session = OddsFeedSessionBuilder.init(dummy, config).setListener(dummy)
                    .setMessageInterest(MessageInterest.SpecifiedMatchesOnly).build();
        MarketDescriptionManager marketManager = session.getMarketDescriptionManager();
        PrintStream out = new PrintStream(new FileOutputStream("market_mappings.txt"));
        for (Market m : marketManager.getMarkets()) {
            printMarket(out, m, includeOutcomes);
        }
        out.close();
        System.exit(0);
    }

    private static void printMarket(PrintStream out, Market m, boolean includeOutcomes) {
        String specifiers = " ";
        for (String s : m.getSpecifiers()) {
            if (specifiers.length() > 1)
                specifiers += "|";
            else
                specifiers = "";
            specifiers += s;
        }

        for (MarketMapping mm : m.getMarketMappings()) {
            String validStr = " ";
            for (SimpleEntry<String, String> e : mm.getValidFor()) {
                if (validStr.length() > 1)
                    validStr += "|";
                else
                    validStr = "";
                validStr += e.getKey() + "=" + e.getValue();
            }
            out.printf("Market;%d;%s;%s;%s;%s;%s;%s;%s\n", m.getId(), m.getMarketDescriptionRaw(), specifiers,
                        mm.getProduct(), mm.getSportId(), validStr, mm.getId(),
                        mm.getSubId() == null ? "" : mm.getSubId());
            if (includeOutcomes) {
                for (Outcome o : m.getOutcomes()) {
                    out.printf("Outcome;%s;%s;%s;%s\n", o.getId(), o.getDescription(), mm.getOutcomeId(o),
                                mm.getOutcomeName(o));
                }
            }
        }
    }

}
