/* Copyright (C) Sportradar AG. See LICENSE for full license governing this code */
package com.sportradar.unifiedodds.restfuloddscache;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import com.sportradar.unifiedodds.sdk.oddsentities.Market;
import com.sportradar.unifiedodds.sdk.oddsentities.MarketStatus;
import com.sportradar.unifiedodds.sdk.oddsentities.Outcome;
import com.sportradar.unifiedodds.sdk.oddsentities.Product;

/**
 * This class represent all the odds for a particular market-line
 */
public class MarketOdds {
	private Map<Outcome, Double> outcomeOdds = new HashMap<>();
	private Map<Outcome, Double> outcomeProbs = new HashMap<>();
	private MarketStatus status;
	private Market market;
	private long statusChanged;
	private Product product;
	private boolean favorite;

	MarketOdds(Market m) {
		market = m;
	}

	public void set(Outcome outcome, double odds) {
		outcomeOdds.put(outcome, odds);
	}

	public void remove(Outcome outcome) {
		outcomeOdds.remove(outcome);
	}

	public void setProbs(Outcome outcome, double probs) {
		outcomeProbs.put(outcome, probs);
	}

	public Double getProbs(Outcome outcome) {
		return outcomeProbs.get(outcome);
	}

	public void removeProbs(Outcome outcome) {
		outcomeProbs.remove(outcome);
	}

	public void setStatus(MarketStatus status, long time, Product product2, boolean fav) {
		this.favorite = fav;
		setStatus(status, time, product2);
	}

	public void setStatus(MarketStatus status, long time, Product product2) {
		if (this.status == MarketStatus.Settled) {
			return;
		}
		statusChanged = time;
		this.product = product2;
		this.status = status;
	}

	public Collection<Entry<Outcome, Double>> getOutcomes() {
		return outcomeOdds.entrySet();
	}

	public Market getMarket() {
		return market;
	}

	public MarketStatus getStatus() {
		return status;
	}

	/**
	 * Called to signal that a certain outcome has been resulted
	 * 
	 * @param outcome
	 * @param refundFactor
	 */
	public void setResult(Outcome outcome, double refundFactor) {
		outcomeOdds.put(outcome, refundFactor);
	}

	/**
	 * @return the time when this market was last updated
	 */
	public long getLastUpdateTime() {
		return this.statusChanged;
	}

	/**
	 * @return the product that updated the odds for this market most recently
	 */
	public Product getLastProduct() {
		return this.product;
	}

	/**
	 * @return true if this market line is the most balanced of all lines
	 */
	public boolean isFavourite() {
		return favorite;
	}
}
